{% extends 'forgeimporters:templates/importer_base.html' %}

{% block importer_fields %}
  <div>
    <label for="project_name">Project name</label>
    <input name="project_name" value="{{ c.form_values['project_name'] }}"/>
    {{ error('project_name') }}
  </div>
{% endblock %}
